/* s518 demo */
/* Sensoray Company 2019 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <pthread.h>
#include <sched.h>
#include <sys/ioctl.h>
#include <time.h>

#include "s518ioctl.h"
#include "s518mid.h"

#define S518_MFG

static char lsdc_text[][100] = {
    "unknown/invalid",//0
    "+/-10V",//1
    "unknown/invalid",//2
    "+/-5V",//3
    "unknown/invalid",//4
    "+/-500mV", //5
    "+/-100mV", //6
    "unknown/invalid",//7
    "Thermocouple E",//8
    "unknown/invalid",//9    
    "Thermocouple J",
    "unknown/invalid",//11
    "Thermocouple K", //12
    "unknown/invalid",
    "Thermocouple T",
    "unknown/invalid",
    "Thermocouple S",
    "unknown/invalid",
    "Thermocouple R",//18
    "unknown/invalid",//19
    "RTD Platinum 100 Ohm 385(0.1C res, full sensor range)",//20
    "unknown/invalid",//21
    "RTD Platinum 100 Ohm 392(0.1C res, full sensor range)",
    "unknown/invalid",//23
    "Thermistor 10k with autorange",//24
    "Current Loop 4-20 mA",//25
    "Strain/pressure gage",//26
    "Resistance 400 Ohm range",//27
    "28 : Resistance 3k Ohm range",//28
    "29 : Resistance 600k Ohm range",//29
    "unknown/invalid",//30
    "Other. Disabled: remove channel from scan list",//31
    "Thermocouple N",//32
    "Thermocouple C",
    "Thermocouple B",//
    "RTD Nickel 120 Ohm",
    "RTD Nickel 200 Ohm",
    "RTD Nickel 1K Ohm (1353 Ohm @60C)",
    "RTD Platinum 100 Ohm 385(0.0125C res, -200:+400C)",
    "RTD Platinum 100 Ohm 392(0.0125C res, -200:+400C)",
    "Thermistor 10k on fixed 600K Ohm range",
    "RTD Copper 10 Ohm",
    "RTD Nickel 1K Ohm (1285 Ohm @60C)",        
    "RTD Platinum 1K Ohm (1422.9 Ohm @110C)"
};



int main( int argc, char *argv[] )
{
    int rc;
    int handle;
    int val;
    struct s518_version ver;
    struct s518_cmd cmd;
    double temp;
    char *arg1;
    
    handle = s518_open(0);
    if (handle == -1) {
        printf( "Middleware/Driver open error\n" );
        return -1;
    }
    rc = ioctl(handle, S518_IOC_VERSION, &ver);
    if (rc == 0) {
        printf("Driver version %d.%d.%d\n", (ver.ver >> 16) & 0xff,
               (ver.ver >> 8) & 0xff, (ver.ver >> 0) & 0xff);
    }
    if (argc < 2) {
        s518_close(handle);
        return 0;
    }

    arg1 = argv[1];
    if (strstr(arg1, "-h") && !strstr(arg1, "-hs")) {
        printf("help\n");
        printf("-h |help\n");
        printf("-r |reset board\n");
        printf("-t |get ambient temperature (requires external sensor)\n");
        printf("-50hz |set 50 Hz rejection mode\n");
        printf("-hs |set high speed mode\n");
        printf("-fwver |get firmware version (from hardware)\n");
        printf("-sd c |get sensor data for channel c\n");
        printf("-sda |get sensor data for all channels\n");
        printf("-cts |get sensor data test\n");
        printf("-lp v |set low power standby (v=0,1)\n");
        printf("-st c t |set sensor type(see -lst command) for channel c\n");
        printf("-fm c m |set fail mode for channel c (m=0 off, m=1 on)\n");
        printf("-flt c v | set LP filter with value v(see manual)\n");
        printf("-lst  |list all sensor types for -st command\n");
#ifdef S518_MFG   
	printf("-1 | read status register\n");
	printf("-2 | read status register loop\n");
	printf("-3 | read firmware loop\n");
        printf("-cal c r v |calibrate channel c with range r, value v(decimal)\n");
        printf("-eeprom | dump eeprom data\n");
        printf("-coeff | coefficient test (calibration)\n");
#endif
        // gage functions
        printf("-gagec c s f | set gage calibration channel c, slope s, oFfset f\n");
        printf("-gageg | get gage calibration channel c\n");
        printf("-gages c v | set gage span to value v, channel c\n");
        printf("-gagez c | zero gage channel c\n");
        printf("-gaget c | tare gage channel c\n");        
        printf("-diag1 |diagnostics only (raw data)\n");
    } else if (strstr(arg1, "-lst")) {
        printf("sensor types or LSDC\n");
        printf("LSDC:logical sensor definition code\n");
        printf("===================================\n");
        printf("1 : Voltage Input +/- 10V\n");
        printf("3 : Voltage Input +/- 5V\n");
        printf("5 : Voltage Input +/- 500mV\n");
        printf("6 : Voltage Input +/- 100mV\n");
        printf("34 : Thermocouple B\n");
        printf("33 : Thermocouple C\n");
        printf("8 : Thermocouple E\n");
        printf("10 : Thermocouple J\n");
        printf("12 : Thermocouple K\n");
        printf("32 : Thermocouple N\n");
        printf("18 : Thermocouple R\n");
        printf("16 : Thermocouple S\n");
        printf("14 : Thermocouple T\n");
        printf("41 : RTD Copper 10 Ohm\n");
        printf("35 : RTD Nickel 120 Ohm\n");
        printf("36 : RTD Nickel 200 Ohm\n");
        printf("37 : RTD Nickel 1K Ohm (1353 Ohm @60C)\n");
        printf("42 : RTD Nickel 1K Ohm (1285 Ohm @60C)\n");        
        printf("20 : RTD Platinum 100 Ohm 385(0.1C res, full sensor range)\n");
        printf("38 : RTD Platinum 100 Ohm 385(0.0125C res, -200:+400C)\n");
        printf("22 : RTD Platinum 100 Ohm 392(0.1C res, full sensor range)\n");
        printf("39 : RTD Platinum 100 Ohm 392(0.0125C res, -200:+400C)\n");
        printf("43 : RTD Platinum 1K Ohm (1422.9 Ohm @110C)\n");
        printf("24 : Thermistor 10k with autorange\n");
        printf("40 : Thermistor 10k on fixed 600K Ohm range\n");
        printf("25 : Current Loop 4-20 mA\n");
        printf("26 : Strain/pressure gage\n");
        printf("27 : Resistance 400 Ohm range\n");
        printf("28 : Resistance 3k Ohm range\n");
        printf("29 : Resistance 600k Ohm range\n");                
        printf("31 : Other. Disabled: remove channel from scan list\n");
        s518_close(handle);
        return 0;
    } else if (strstr(arg1, "-cal")) {
        int c, r;
        double v;
        double cval;
        if (argc < 5) {
            printf("must specify c, r, v\n");
            s518_close(handle);
            return -EINVAL;
        }        
        c = atoi(argv[2]); // channel
        r = atoi(argv[3]); // range
        v = atof(argv[4]);
        rc = s518_calibrate(handle, c, r, v, &cval);
        if (rc != 0) {
            printf("calibration failed\n");
            return -EINVAL;
        }
        printf("518 calibration channel %d, range %d, value %lf, cal val %lf\n",
               c, r, v, cval);
    } else if (strstr(arg1, "-eeprom")) {
        double eeprom[7];
        int i;
        rc = s518_get_eeprom_data(handle, eeprom);
        if (rc != 0) {
            printf("calibration failed\n");
            return -EINVAL;
        }
        for (i = 0; i < 8; i++) {
            printf("[%d]eeprom_data %lf\n", i, eeprom[i]);
        }
    } else if (strstr(arg1, "-coeff")) {
      rc = s518_set_coefficients(handle, 5, 0.0, 1.0, 0.0);
        if (rc != 0) {
            printf("coefficients failed\n");
            return -EINVAL;
        }
	printf("cofficients\n");
  } else if (strstr(arg1, "-t")) {
        rc = s518_get_ambient_temp(handle, &temp);
        if (rc != 0) {
            printf("failed to get ambient temp\n");
            s518_close(handle);
            return rc;
        }
        printf("ambient temperature %lf degrees C\n", temp);        
    } else if (strstr(arg1, "-r")) {
        printf("resetting board\n");
        rc = s518_reset_board(handle);
        if (rc != 0) {
            printf("failed to reset board\n");
            s518_close(handle);            
            return rc;
        }
    } else if (strstr(arg1, "-lp")) {
        int mode;
        if (argc < 3) {
            printf("must specify value\n");
            s518_close(handle);
            return -EINVAL;
        }
        mode = atoi(argv[2]);

        rc = s518_set_low_pwr_standby(handle, mode);
        if (rc != 0) {
            printf("failed to reset board\n");
            s518_close(handle);            
            return rc;
        }
        if (mode)
            printf("Setting low power standby\n");
        else
            printf("Turning off low power standby(wakeup)\n");
    } else if (strstr(arg1, "-50hz")) {
        printf("setting 50 Hz rejection\n");
        rc = s518_set_reject_50hz(handle);
        if (rc != 0) {
            printf("failed to set 50hz rejection\n");
            s518_close(handle);            
            return rc;
        }
    } else if (strstr(arg1, "-hs")) {
        printf("setting high speed mode\n");
        rc = s518_set_hi_speed_mode(handle);
        if (rc != 0) {
            printf("failed to set high speed mode\n");
            s518_close(handle);            
            return rc;
        }
    } else if (strstr(arg1, "-fwver")) {
        short fwver;
        rc = s518_get_fw_version(handle, &fwver);
        if (rc != 0) {
            printf("failed to get firmware version\n");
            s518_close(handle);            
            return rc;
        }
        printf("HW firmware version %f\n", fwver/100.0);
    } else if (strstr(arg1, "-sda")) {
        double sdata[NUM_518_CHANNELS];
        int i;
        rc = s518_get_all_sensors(handle, sdata);
        if (rc != 0) {
            printf("failed to get sensor data\n");
            s518_close(handle);            
            return rc;
        }
        for (i = 0; i < NUM_518_CHANNELS; i++) {
            printf("sensor data[%d]: %lf\n", i, sdata[i]);
        }
    } else if (strstr(arg1, "-cts")) {
        double sdata[NUM_518_CHANNELS];
        int i;
	while (1) {
	  rc = s518_get_all_sensors(handle, sdata);
	  //rc = s518_get_sensor_data(handle, 0, sdata);
	  if (rc != 0) {
            printf("failed to get sensor data\n");
            continue;//
	  }
	  for (i = 0; i < NUM_518_CHANNELS; i++) {
            printf("sensor data[%d]: %lf\n", i, sdata[i]);
	  }
	  if (sdata[0] < 4.0 || sdata[0] > 7.1) {
	    printf("out of range channel 0\n");
	    break;
	  }
	  printf("\n");
	  usleep(3*1000);
	}
    } else if (strstr(arg1, "-diag1")) {
        short sdata[NUM_518_CHANNELS];
        int i;
        rc = s518_diag_func1(handle, sdata);
        if (rc != 0) {
            printf("failed to get sensor data\n");
            s518_close(handle);            
            return rc;
        }
        for (i = 0; i < NUM_518_CHANNELS; i++) {
            printf("sensor data[%d]: %d, hex:0x%x\n", i, sdata[i], sdata[i]);
        }        
    } else if (strstr(arg1, "-1")) {
        short fwver;
        double sdata;
        int ch;
	struct s518_cmd cmd;

        cmd.val = 0;
        rc = ioctl(handle, S518_IOC_READ_STATUS, &cmd);
	if (rc == 0)
	  printf("read status 0x%x\n", cmd.val);
	else
	  printf("failed read status\n");
	return 0;
    } else if (strstr(arg1, "-2")) {
        short fwver;
        double sdata;
        int ch;
	struct s518_cmd cmd;
	while (1) {
	  cmd.val = 0;
	  rc = ioctl(handle, S518_IOC_READ_STATUS, &cmd);
	  if (rc == 0)
	    printf("read status 0x%x\n", cmd.val);
	  else
	    printf("failed read status\n");
	  if (cmd.val & 0x10) {
	    printf("fault detected !\n");
	    return 0;
	  }
	}
	return 0;
    } else if (strstr(arg1, "-3")) {
        short fwver;
	int flags;
	while (1) {
	  rc = s518_get_fw_version(handle, &fwver);
	  if (rc == 0)
	    printf("read firmware %lf\n", fwver/100.0);
	  else
	    printf("failed read status\n");
	  rc = ioctl(handle, S518_IOC_GET_FAULTFLAGS, &flags);
	  if (rc == 0) {
	    if (flags)
	      printf("fault flags 0x%x\n", flags);
	    if (flags & 0x10) {
	      printf("fault detected !\n");
	      return 0;
	    }
	  } else
	    printf("failed read status\n");
	}
	return 0;
    } else if (strstr(arg1, "-sd")) {
        short fwver;
        double sdata;
        int ch;
        if (argc < 3) {
            printf("no channel specified using channel 0\n");
            ch = 0;
        } else
            ch = atoi(argv[2]);
            
        rc = s518_get_sensor_data(handle, ch, &sdata);
        if (rc != 0) {
            printf("Failed to get sensor data for channel %d\n", ch);
            s518_close(handle);            
            return rc;
        }
        printf("sensor data[%d]: %lf\n", ch, sdata);
    } else if (strstr(arg1, "-fm")) {
        short fwver;
        double sdata;
        int mode;
        int ch;
        if (argc < 4) {
            printf("must specify channel and mode 0 or 1\n");
            s518_close(handle);
            return -EINVAL;
        }
        ch = atoi(argv[2]);        
        mode = atoi(argv[3]);
        rc = s518_set_fail_mode(handle, ch, mode);
        if (rc != 0) {
            printf("Failed to get set fail mode for channel %d\n", ch);
            s518_close(handle);            
            return rc;
        }
        if (mode)
            printf("Setting fail hi mode for channel %d\n", ch);
        else
            printf("Turning off fail hi mode for channel %d\n", ch);
    } else if (strstr(arg1, "-flt")) {
        short fwver;
        int c, v;
        if (argc < 4) {
            printf("must specify channel and filter value\n");
            s518_close(handle);
            return -EINVAL;
        }
        c = atoi(argv[2]);        
        v = atoi(argv[3]);
        rc = s518_set_filter(handle, c, v);
        if (rc != 0) {
            printf("Failed to get set filter for channel %d\n", c);
            s518_close(handle);            
            return rc;
        }
        printf("518: set filter value %d for channel %d\n", v, c);
    } else if (strstr(arg1, "-st")) {
        short fwver;
        double sdata;
        int type;
        int ch;
        if (argc < 4) {
            printf("must specify channel and type (see -lst command)\n");
            s518_close(handle);
            return -EINVAL;
        }
        ch = atoi(argv[2]);        
        type = atoi(argv[3]);
        if (type >= 44) {
            printf("invalid sensor type\n");
            s518_close(handle);
            return -EINVAL;
        }
        rc = s518_set_sensor_type(handle, ch, type);
        if (rc != 0) {
            printf("Failed to get set sensor type for channel %d\n", ch);
            s518_close(handle);            
            return rc;
        }
        printf("Set channel %d to sensor type %d [%s]\n", ch, type,
               lsdc_text[type]);
    } else if (strstr(arg1, "-gagec")) {
        int c;
        double slope;
        short offset;
        if (argc < 5) {
            printf("must specify channel, slope, offset\n");
            s518_close(handle);
            return -EINVAL;
        }
        c = atoi(argv[2]);
        slope = atof(argv[3]);
        offset = (short) atoi(argv[4]);
        rc = s518_set_gage_cal(handle, c, slope, offset);
        if (rc != 0) {
            printf("Failed to get set gage calibration for channel %d\n", c);
            s518_close(handle);            
            return rc;
        }
        printf("518: set gage calibration channel %d, slope %lf, offset %d\n",
               c, slope, offset);
    } else if (strstr(arg1, "-gageg")) {
        int c;
        double slope;
        short offset;
        if (argc < 3) {
            printf("must specify channel\n");
            s518_close(handle);
            return -EINVAL;
        }
        c = atoi(argv[2]);
        rc = s518_get_gage_cal(handle, c, &slope, &offset);
        if (rc != 0) {
            printf("Failed to get gage calibration for channel %d\n", c);
            s518_close(handle);            
            return rc;
        }
        printf("518: gage calibration channel %d, slope %lf, offset %d\n",
               c, slope, offset);
    } else if (strstr(arg1, "-gages")) {
        int c;
        int span;
        if (argc < 4) {
            printf("must specify channel and gage span\n");
            s518_close(handle);
            return -EINVAL;
        }
        c = atoi(argv[2]);
        span = atoi(argv[3]);
        rc = s518_set_gage_span(handle, c, span);
        if (rc != 0) {
            printf("Failed to set gage span for channel %d\n", c);
            s518_close(handle);            
            return rc;
        }
        printf("518: set gage span %d for channel %d\n", span, c);
    } else if (strstr(arg1, "-gagez")) {
        int c;
        int span;
        if (argc < 3) {
            printf("must specify channel\n");
            s518_close(handle);
            return -EINVAL;
        }
        c = atoi(argv[2]);

        rc = s518_set_gage_zero(handle, c);
        if (rc != 0) {
            printf("Failed to set gage zero for channel %d\n", c);
            s518_close(handle);            
            return rc;
        }
        printf("518: set gage zero for channel %d\n", c);
    } else if (strstr(arg1, "-gaget")) {
        int c;
        int span;
        if (argc < 3) {
            printf("must specify channel\n");
            s518_close(handle);
            return -EINVAL;
        }
        c = atoi(argv[2]);
        rc = s518_set_gage_tare(handle, c);
        if (rc != 0) {
            printf("Failed to set gage tare for channel %d\n", c);
            s518_close(handle);            
            return rc;
        }
        printf("518: set gage tare for channel %d\n", c);
    }
    s518_close(handle);
    return 0;
}
